# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc
import warnings

from . import minio_service_pb2 as minio__service__pb2

GRPC_GENERATED_VERSION = '1.75.1'
GRPC_VERSION = grpc.__version__
_version_not_supported = False

try:
    from grpc._utilities import first_version_is_lower
    _version_not_supported = first_version_is_lower(GRPC_VERSION, GRPC_GENERATED_VERSION)
except ImportError:
    _version_not_supported = True

if _version_not_supported:
    raise RuntimeError(
        f'The grpc package installed is at version {GRPC_VERSION},'
        + f' but the generated code in minio_service_pb2_grpc.py depends on'
        + f' grpcio>={GRPC_GENERATED_VERSION}.'
        + f' Please upgrade your grpc module to grpcio>={GRPC_GENERATED_VERSION}'
        + f' or downgrade your generated code using grpcio-tools<={GRPC_VERSION}.'
    )


class MinIOServiceStub(object):
    """========================================
    MinIO 对象存储服务
    ========================================
    这是一个通用的对象存储服务，为所有后端服务提供文件存储能力
    支持：
    - 用户和组织级别的隔离
    - 桶（Bucket）和对象（Object）管理
    - 流式上传/下载（支持大文件）
    - 预签名 URL（临时访问链接）
    - 对象元数据管理

    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateBucket = channel.unary_unary(
                '/isa.minio.MinIOService/CreateBucket',
                request_serializer=minio__service__pb2.CreateBucketRequest.SerializeToString,
                response_deserializer=minio__service__pb2.CreateBucketResponse.FromString,
                _registered_method=True)
        self.ListBuckets = channel.unary_unary(
                '/isa.minio.MinIOService/ListBuckets',
                request_serializer=minio__service__pb2.ListBucketsRequest.SerializeToString,
                response_deserializer=minio__service__pb2.ListBucketsResponse.FromString,
                _registered_method=True)
        self.DeleteBucket = channel.unary_unary(
                '/isa.minio.MinIOService/DeleteBucket',
                request_serializer=minio__service__pb2.DeleteBucketRequest.SerializeToString,
                response_deserializer=minio__service__pb2.DeleteBucketResponse.FromString,
                _registered_method=True)
        self.GetBucketInfo = channel.unary_unary(
                '/isa.minio.MinIOService/GetBucketInfo',
                request_serializer=minio__service__pb2.GetBucketInfoRequest.SerializeToString,
                response_deserializer=minio__service__pb2.GetBucketInfoResponse.FromString,
                _registered_method=True)
        self.SetBucketPolicy = channel.unary_unary(
                '/isa.minio.MinIOService/SetBucketPolicy',
                request_serializer=minio__service__pb2.SetBucketPolicyRequest.SerializeToString,
                response_deserializer=minio__service__pb2.SetBucketPolicyResponse.FromString,
                _registered_method=True)
        self.GetBucketPolicy = channel.unary_unary(
                '/isa.minio.MinIOService/GetBucketPolicy',
                request_serializer=minio__service__pb2.GetBucketPolicyRequest.SerializeToString,
                response_deserializer=minio__service__pb2.GetBucketPolicyResponse.FromString,
                _registered_method=True)
        self.SetBucketTags = channel.unary_unary(
                '/isa.minio.MinIOService/SetBucketTags',
                request_serializer=minio__service__pb2.SetBucketTagsRequest.SerializeToString,
                response_deserializer=minio__service__pb2.SetBucketTagsResponse.FromString,
                _registered_method=True)
        self.GetBucketTags = channel.unary_unary(
                '/isa.minio.MinIOService/GetBucketTags',
                request_serializer=minio__service__pb2.GetBucketTagsRequest.SerializeToString,
                response_deserializer=minio__service__pb2.GetBucketTagsResponse.FromString,
                _registered_method=True)
        self.DeleteBucketTags = channel.unary_unary(
                '/isa.minio.MinIOService/DeleteBucketTags',
                request_serializer=minio__service__pb2.DeleteBucketTagsRequest.SerializeToString,
                response_deserializer=minio__service__pb2.DeleteBucketTagsResponse.FromString,
                _registered_method=True)
        self.SetBucketVersioning = channel.unary_unary(
                '/isa.minio.MinIOService/SetBucketVersioning',
                request_serializer=minio__service__pb2.SetBucketVersioningRequest.SerializeToString,
                response_deserializer=minio__service__pb2.SetBucketVersioningResponse.FromString,
                _registered_method=True)
        self.GetBucketVersioning = channel.unary_unary(
                '/isa.minio.MinIOService/GetBucketVersioning',
                request_serializer=minio__service__pb2.GetBucketVersioningRequest.SerializeToString,
                response_deserializer=minio__service__pb2.GetBucketVersioningResponse.FromString,
                _registered_method=True)
        self.SetBucketLifecycle = channel.unary_unary(
                '/isa.minio.MinIOService/SetBucketLifecycle',
                request_serializer=minio__service__pb2.SetBucketLifecycleRequest.SerializeToString,
                response_deserializer=minio__service__pb2.SetBucketLifecycleResponse.FromString,
                _registered_method=True)
        self.GetBucketLifecycle = channel.unary_unary(
                '/isa.minio.MinIOService/GetBucketLifecycle',
                request_serializer=minio__service__pb2.GetBucketLifecycleRequest.SerializeToString,
                response_deserializer=minio__service__pb2.GetBucketLifecycleResponse.FromString,
                _registered_method=True)
        self.DeleteBucketLifecycle = channel.unary_unary(
                '/isa.minio.MinIOService/DeleteBucketLifecycle',
                request_serializer=minio__service__pb2.DeleteBucketLifecycleRequest.SerializeToString,
                response_deserializer=minio__service__pb2.DeleteBucketLifecycleResponse.FromString,
                _registered_method=True)
        self.PutObject = channel.stream_unary(
                '/isa.minio.MinIOService/PutObject',
                request_serializer=minio__service__pb2.PutObjectRequest.SerializeToString,
                response_deserializer=minio__service__pb2.PutObjectResponse.FromString,
                _registered_method=True)
        self.GetObject = channel.unary_stream(
                '/isa.minio.MinIOService/GetObject',
                request_serializer=minio__service__pb2.GetObjectRequest.SerializeToString,
                response_deserializer=minio__service__pb2.GetObjectResponse.FromString,
                _registered_method=True)
        self.DeleteObject = channel.unary_unary(
                '/isa.minio.MinIOService/DeleteObject',
                request_serializer=minio__service__pb2.DeleteObjectRequest.SerializeToString,
                response_deserializer=minio__service__pb2.DeleteObjectResponse.FromString,
                _registered_method=True)
        self.DeleteObjects = channel.unary_unary(
                '/isa.minio.MinIOService/DeleteObjects',
                request_serializer=minio__service__pb2.DeleteObjectsRequest.SerializeToString,
                response_deserializer=minio__service__pb2.DeleteObjectsResponse.FromString,
                _registered_method=True)
        self.ListObjects = channel.unary_unary(
                '/isa.minio.MinIOService/ListObjects',
                request_serializer=minio__service__pb2.ListObjectsRequest.SerializeToString,
                response_deserializer=minio__service__pb2.ListObjectsResponse.FromString,
                _registered_method=True)
        self.CopyObject = channel.unary_unary(
                '/isa.minio.MinIOService/CopyObject',
                request_serializer=minio__service__pb2.CopyObjectRequest.SerializeToString,
                response_deserializer=minio__service__pb2.CopyObjectResponse.FromString,
                _registered_method=True)
        self.StatObject = channel.unary_unary(
                '/isa.minio.MinIOService/StatObject',
                request_serializer=minio__service__pb2.StatObjectRequest.SerializeToString,
                response_deserializer=minio__service__pb2.StatObjectResponse.FromString,
                _registered_method=True)
        self.SetObjectTags = channel.unary_unary(
                '/isa.minio.MinIOService/SetObjectTags',
                request_serializer=minio__service__pb2.SetObjectTagsRequest.SerializeToString,
                response_deserializer=minio__service__pb2.SetObjectTagsResponse.FromString,
                _registered_method=True)
        self.GetObjectTags = channel.unary_unary(
                '/isa.minio.MinIOService/GetObjectTags',
                request_serializer=minio__service__pb2.GetObjectTagsRequest.SerializeToString,
                response_deserializer=minio__service__pb2.GetObjectTagsResponse.FromString,
                _registered_method=True)
        self.DeleteObjectTags = channel.unary_unary(
                '/isa.minio.MinIOService/DeleteObjectTags',
                request_serializer=minio__service__pb2.DeleteObjectTagsRequest.SerializeToString,
                response_deserializer=minio__service__pb2.DeleteObjectTagsResponse.FromString,
                _registered_method=True)
        self.GetPresignedURL = channel.unary_unary(
                '/isa.minio.MinIOService/GetPresignedURL',
                request_serializer=minio__service__pb2.GetPresignedURLRequest.SerializeToString,
                response_deserializer=minio__service__pb2.GetPresignedURLResponse.FromString,
                _registered_method=True)
        self.GetPresignedPutURL = channel.unary_unary(
                '/isa.minio.MinIOService/GetPresignedPutURL',
                request_serializer=minio__service__pb2.GetPresignedPutURLRequest.SerializeToString,
                response_deserializer=minio__service__pb2.GetPresignedPutURLResponse.FromString,
                _registered_method=True)
        self.InitiateMultipartUpload = channel.unary_unary(
                '/isa.minio.MinIOService/InitiateMultipartUpload',
                request_serializer=minio__service__pb2.InitiateMultipartUploadRequest.SerializeToString,
                response_deserializer=minio__service__pb2.InitiateMultipartUploadResponse.FromString,
                _registered_method=True)
        self.UploadPart = channel.stream_unary(
                '/isa.minio.MinIOService/UploadPart',
                request_serializer=minio__service__pb2.UploadPartRequest.SerializeToString,
                response_deserializer=minio__service__pb2.UploadPartResponse.FromString,
                _registered_method=True)
        self.CompleteMultipartUpload = channel.unary_unary(
                '/isa.minio.MinIOService/CompleteMultipartUpload',
                request_serializer=minio__service__pb2.CompleteMultipartUploadRequest.SerializeToString,
                response_deserializer=minio__service__pb2.CompleteMultipartUploadResponse.FromString,
                _registered_method=True)
        self.AbortMultipartUpload = channel.unary_unary(
                '/isa.minio.MinIOService/AbortMultipartUpload',
                request_serializer=minio__service__pb2.AbortMultipartUploadRequest.SerializeToString,
                response_deserializer=minio__service__pb2.AbortMultipartUploadResponse.FromString,
                _registered_method=True)
        self.HealthCheck = channel.unary_unary(
                '/isa.minio.MinIOService/HealthCheck',
                request_serializer=minio__service__pb2.MinIOHealthCheckRequest.SerializeToString,
                response_deserializer=minio__service__pb2.MinIOHealthCheckResponse.FromString,
                _registered_method=True)


class MinIOServiceServicer(object):
    """========================================
    MinIO 对象存储服务
    ========================================
    这是一个通用的对象存储服务，为所有后端服务提供文件存储能力
    支持：
    - 用户和组织级别的隔离
    - 桶（Bucket）和对象（Object）管理
    - 流式上传/下载（支持大文件）
    - 预签名 URL（临时访问链接）
    - 对象元数据管理

    """

    def CreateBucket(self, request, context):
        """========== 桶管理 ==========
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListBuckets(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteBucket(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBucketInfo(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetBucketPolicy(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBucketPolicy(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetBucketTags(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBucketTags(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteBucketTags(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetBucketVersioning(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBucketVersioning(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetBucketLifecycle(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetBucketLifecycle(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteBucketLifecycle(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def PutObject(self, request_iterator, context):
        """========== 对象操作 ==========
        流式上传（支持大文件分块上传）
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetObject(self, request, context):
        """流式下载（支持大文件分块下载）
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteObject(self, request, context):
        """删除对象
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteObjects(self, request, context):
        """批量删除对象
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListObjects(self, request, context):
        """列出对象
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CopyObject(self, request, context):
        """复制对象
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StatObject(self, request, context):
        """获取对象信息（不下载内容）
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetObjectTags(self, request, context):
        """对象标签管理
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetObjectTags(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteObjectTags(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPresignedURL(self, request, context):
        """========== 预签名 URL ==========
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPresignedPutURL(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def InitiateMultipartUpload(self, request, context):
        """========== 多部分上传 ==========
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UploadPart(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CompleteMultipartUpload(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AbortMultipartUpload(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def HealthCheck(self, request, context):
        """========== 健康检查 ==========
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MinIOServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateBucket': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateBucket,
                    request_deserializer=minio__service__pb2.CreateBucketRequest.FromString,
                    response_serializer=minio__service__pb2.CreateBucketResponse.SerializeToString,
            ),
            'ListBuckets': grpc.unary_unary_rpc_method_handler(
                    servicer.ListBuckets,
                    request_deserializer=minio__service__pb2.ListBucketsRequest.FromString,
                    response_serializer=minio__service__pb2.ListBucketsResponse.SerializeToString,
            ),
            'DeleteBucket': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteBucket,
                    request_deserializer=minio__service__pb2.DeleteBucketRequest.FromString,
                    response_serializer=minio__service__pb2.DeleteBucketResponse.SerializeToString,
            ),
            'GetBucketInfo': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBucketInfo,
                    request_deserializer=minio__service__pb2.GetBucketInfoRequest.FromString,
                    response_serializer=minio__service__pb2.GetBucketInfoResponse.SerializeToString,
            ),
            'SetBucketPolicy': grpc.unary_unary_rpc_method_handler(
                    servicer.SetBucketPolicy,
                    request_deserializer=minio__service__pb2.SetBucketPolicyRequest.FromString,
                    response_serializer=minio__service__pb2.SetBucketPolicyResponse.SerializeToString,
            ),
            'GetBucketPolicy': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBucketPolicy,
                    request_deserializer=minio__service__pb2.GetBucketPolicyRequest.FromString,
                    response_serializer=minio__service__pb2.GetBucketPolicyResponse.SerializeToString,
            ),
            'SetBucketTags': grpc.unary_unary_rpc_method_handler(
                    servicer.SetBucketTags,
                    request_deserializer=minio__service__pb2.SetBucketTagsRequest.FromString,
                    response_serializer=minio__service__pb2.SetBucketTagsResponse.SerializeToString,
            ),
            'GetBucketTags': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBucketTags,
                    request_deserializer=minio__service__pb2.GetBucketTagsRequest.FromString,
                    response_serializer=minio__service__pb2.GetBucketTagsResponse.SerializeToString,
            ),
            'DeleteBucketTags': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteBucketTags,
                    request_deserializer=minio__service__pb2.DeleteBucketTagsRequest.FromString,
                    response_serializer=minio__service__pb2.DeleteBucketTagsResponse.SerializeToString,
            ),
            'SetBucketVersioning': grpc.unary_unary_rpc_method_handler(
                    servicer.SetBucketVersioning,
                    request_deserializer=minio__service__pb2.SetBucketVersioningRequest.FromString,
                    response_serializer=minio__service__pb2.SetBucketVersioningResponse.SerializeToString,
            ),
            'GetBucketVersioning': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBucketVersioning,
                    request_deserializer=minio__service__pb2.GetBucketVersioningRequest.FromString,
                    response_serializer=minio__service__pb2.GetBucketVersioningResponse.SerializeToString,
            ),
            'SetBucketLifecycle': grpc.unary_unary_rpc_method_handler(
                    servicer.SetBucketLifecycle,
                    request_deserializer=minio__service__pb2.SetBucketLifecycleRequest.FromString,
                    response_serializer=minio__service__pb2.SetBucketLifecycleResponse.SerializeToString,
            ),
            'GetBucketLifecycle': grpc.unary_unary_rpc_method_handler(
                    servicer.GetBucketLifecycle,
                    request_deserializer=minio__service__pb2.GetBucketLifecycleRequest.FromString,
                    response_serializer=minio__service__pb2.GetBucketLifecycleResponse.SerializeToString,
            ),
            'DeleteBucketLifecycle': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteBucketLifecycle,
                    request_deserializer=minio__service__pb2.DeleteBucketLifecycleRequest.FromString,
                    response_serializer=minio__service__pb2.DeleteBucketLifecycleResponse.SerializeToString,
            ),
            'PutObject': grpc.stream_unary_rpc_method_handler(
                    servicer.PutObject,
                    request_deserializer=minio__service__pb2.PutObjectRequest.FromString,
                    response_serializer=minio__service__pb2.PutObjectResponse.SerializeToString,
            ),
            'GetObject': grpc.unary_stream_rpc_method_handler(
                    servicer.GetObject,
                    request_deserializer=minio__service__pb2.GetObjectRequest.FromString,
                    response_serializer=minio__service__pb2.GetObjectResponse.SerializeToString,
            ),
            'DeleteObject': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteObject,
                    request_deserializer=minio__service__pb2.DeleteObjectRequest.FromString,
                    response_serializer=minio__service__pb2.DeleteObjectResponse.SerializeToString,
            ),
            'DeleteObjects': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteObjects,
                    request_deserializer=minio__service__pb2.DeleteObjectsRequest.FromString,
                    response_serializer=minio__service__pb2.DeleteObjectsResponse.SerializeToString,
            ),
            'ListObjects': grpc.unary_unary_rpc_method_handler(
                    servicer.ListObjects,
                    request_deserializer=minio__service__pb2.ListObjectsRequest.FromString,
                    response_serializer=minio__service__pb2.ListObjectsResponse.SerializeToString,
            ),
            'CopyObject': grpc.unary_unary_rpc_method_handler(
                    servicer.CopyObject,
                    request_deserializer=minio__service__pb2.CopyObjectRequest.FromString,
                    response_serializer=minio__service__pb2.CopyObjectResponse.SerializeToString,
            ),
            'StatObject': grpc.unary_unary_rpc_method_handler(
                    servicer.StatObject,
                    request_deserializer=minio__service__pb2.StatObjectRequest.FromString,
                    response_serializer=minio__service__pb2.StatObjectResponse.SerializeToString,
            ),
            'SetObjectTags': grpc.unary_unary_rpc_method_handler(
                    servicer.SetObjectTags,
                    request_deserializer=minio__service__pb2.SetObjectTagsRequest.FromString,
                    response_serializer=minio__service__pb2.SetObjectTagsResponse.SerializeToString,
            ),
            'GetObjectTags': grpc.unary_unary_rpc_method_handler(
                    servicer.GetObjectTags,
                    request_deserializer=minio__service__pb2.GetObjectTagsRequest.FromString,
                    response_serializer=minio__service__pb2.GetObjectTagsResponse.SerializeToString,
            ),
            'DeleteObjectTags': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteObjectTags,
                    request_deserializer=minio__service__pb2.DeleteObjectTagsRequest.FromString,
                    response_serializer=minio__service__pb2.DeleteObjectTagsResponse.SerializeToString,
            ),
            'GetPresignedURL': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPresignedURL,
                    request_deserializer=minio__service__pb2.GetPresignedURLRequest.FromString,
                    response_serializer=minio__service__pb2.GetPresignedURLResponse.SerializeToString,
            ),
            'GetPresignedPutURL': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPresignedPutURL,
                    request_deserializer=minio__service__pb2.GetPresignedPutURLRequest.FromString,
                    response_serializer=minio__service__pb2.GetPresignedPutURLResponse.SerializeToString,
            ),
            'InitiateMultipartUpload': grpc.unary_unary_rpc_method_handler(
                    servicer.InitiateMultipartUpload,
                    request_deserializer=minio__service__pb2.InitiateMultipartUploadRequest.FromString,
                    response_serializer=minio__service__pb2.InitiateMultipartUploadResponse.SerializeToString,
            ),
            'UploadPart': grpc.stream_unary_rpc_method_handler(
                    servicer.UploadPart,
                    request_deserializer=minio__service__pb2.UploadPartRequest.FromString,
                    response_serializer=minio__service__pb2.UploadPartResponse.SerializeToString,
            ),
            'CompleteMultipartUpload': grpc.unary_unary_rpc_method_handler(
                    servicer.CompleteMultipartUpload,
                    request_deserializer=minio__service__pb2.CompleteMultipartUploadRequest.FromString,
                    response_serializer=minio__service__pb2.CompleteMultipartUploadResponse.SerializeToString,
            ),
            'AbortMultipartUpload': grpc.unary_unary_rpc_method_handler(
                    servicer.AbortMultipartUpload,
                    request_deserializer=minio__service__pb2.AbortMultipartUploadRequest.FromString,
                    response_serializer=minio__service__pb2.AbortMultipartUploadResponse.SerializeToString,
            ),
            'HealthCheck': grpc.unary_unary_rpc_method_handler(
                    servicer.HealthCheck,
                    request_deserializer=minio__service__pb2.MinIOHealthCheckRequest.FromString,
                    response_serializer=minio__service__pb2.MinIOHealthCheckResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'isa.minio.MinIOService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))
    server.add_registered_method_handlers('isa.minio.MinIOService', rpc_method_handlers)


 # This class is part of an EXPERIMENTAL API.
class MinIOService(object):
    """========================================
    MinIO 对象存储服务
    ========================================
    这是一个通用的对象存储服务，为所有后端服务提供文件存储能力
    支持：
    - 用户和组织级别的隔离
    - 桶（Bucket）和对象（Object）管理
    - 流式上传/下载（支持大文件）
    - 预签名 URL（临时访问链接）
    - 对象元数据管理

    """

    @staticmethod
    def CreateBucket(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/CreateBucket',
            minio__service__pb2.CreateBucketRequest.SerializeToString,
            minio__service__pb2.CreateBucketResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListBuckets(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/ListBuckets',
            minio__service__pb2.ListBucketsRequest.SerializeToString,
            minio__service__pb2.ListBucketsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteBucket(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/DeleteBucket',
            minio__service__pb2.DeleteBucketRequest.SerializeToString,
            minio__service__pb2.DeleteBucketResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetBucketInfo(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/GetBucketInfo',
            minio__service__pb2.GetBucketInfoRequest.SerializeToString,
            minio__service__pb2.GetBucketInfoResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetBucketPolicy(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/SetBucketPolicy',
            minio__service__pb2.SetBucketPolicyRequest.SerializeToString,
            minio__service__pb2.SetBucketPolicyResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetBucketPolicy(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/GetBucketPolicy',
            minio__service__pb2.GetBucketPolicyRequest.SerializeToString,
            minio__service__pb2.GetBucketPolicyResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetBucketTags(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/SetBucketTags',
            minio__service__pb2.SetBucketTagsRequest.SerializeToString,
            minio__service__pb2.SetBucketTagsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetBucketTags(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/GetBucketTags',
            minio__service__pb2.GetBucketTagsRequest.SerializeToString,
            minio__service__pb2.GetBucketTagsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteBucketTags(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/DeleteBucketTags',
            minio__service__pb2.DeleteBucketTagsRequest.SerializeToString,
            minio__service__pb2.DeleteBucketTagsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetBucketVersioning(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/SetBucketVersioning',
            minio__service__pb2.SetBucketVersioningRequest.SerializeToString,
            minio__service__pb2.SetBucketVersioningResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetBucketVersioning(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/GetBucketVersioning',
            minio__service__pb2.GetBucketVersioningRequest.SerializeToString,
            minio__service__pb2.GetBucketVersioningResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetBucketLifecycle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/SetBucketLifecycle',
            minio__service__pb2.SetBucketLifecycleRequest.SerializeToString,
            minio__service__pb2.SetBucketLifecycleResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetBucketLifecycle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/GetBucketLifecycle',
            minio__service__pb2.GetBucketLifecycleRequest.SerializeToString,
            minio__service__pb2.GetBucketLifecycleResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteBucketLifecycle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/DeleteBucketLifecycle',
            minio__service__pb2.DeleteBucketLifecycleRequest.SerializeToString,
            minio__service__pb2.DeleteBucketLifecycleResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def PutObject(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(
            request_iterator,
            target,
            '/isa.minio.MinIOService/PutObject',
            minio__service__pb2.PutObjectRequest.SerializeToString,
            minio__service__pb2.PutObjectResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetObject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(
            request,
            target,
            '/isa.minio.MinIOService/GetObject',
            minio__service__pb2.GetObjectRequest.SerializeToString,
            minio__service__pb2.GetObjectResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteObject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/DeleteObject',
            minio__service__pb2.DeleteObjectRequest.SerializeToString,
            minio__service__pb2.DeleteObjectResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteObjects(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/DeleteObjects',
            minio__service__pb2.DeleteObjectsRequest.SerializeToString,
            minio__service__pb2.DeleteObjectsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def ListObjects(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/ListObjects',
            minio__service__pb2.ListObjectsRequest.SerializeToString,
            minio__service__pb2.ListObjectsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CopyObject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/CopyObject',
            minio__service__pb2.CopyObjectRequest.SerializeToString,
            minio__service__pb2.CopyObjectResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def StatObject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/StatObject',
            minio__service__pb2.StatObjectRequest.SerializeToString,
            minio__service__pb2.StatObjectResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def SetObjectTags(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/SetObjectTags',
            minio__service__pb2.SetObjectTagsRequest.SerializeToString,
            minio__service__pb2.SetObjectTagsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetObjectTags(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/GetObjectTags',
            minio__service__pb2.GetObjectTagsRequest.SerializeToString,
            minio__service__pb2.GetObjectTagsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def DeleteObjectTags(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/DeleteObjectTags',
            minio__service__pb2.DeleteObjectTagsRequest.SerializeToString,
            minio__service__pb2.DeleteObjectTagsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPresignedURL(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/GetPresignedURL',
            minio__service__pb2.GetPresignedURLRequest.SerializeToString,
            minio__service__pb2.GetPresignedURLResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def GetPresignedPutURL(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/GetPresignedPutURL',
            minio__service__pb2.GetPresignedPutURLRequest.SerializeToString,
            minio__service__pb2.GetPresignedPutURLResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def InitiateMultipartUpload(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/InitiateMultipartUpload',
            minio__service__pb2.InitiateMultipartUploadRequest.SerializeToString,
            minio__service__pb2.InitiateMultipartUploadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def UploadPart(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_unary(
            request_iterator,
            target,
            '/isa.minio.MinIOService/UploadPart',
            minio__service__pb2.UploadPartRequest.SerializeToString,
            minio__service__pb2.UploadPartResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def CompleteMultipartUpload(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/CompleteMultipartUpload',
            minio__service__pb2.CompleteMultipartUploadRequest.SerializeToString,
            minio__service__pb2.CompleteMultipartUploadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def AbortMultipartUpload(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/AbortMultipartUpload',
            minio__service__pb2.AbortMultipartUploadRequest.SerializeToString,
            minio__service__pb2.AbortMultipartUploadResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)

    @staticmethod
    def HealthCheck(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(
            request,
            target,
            '/isa.minio.MinIOService/HealthCheck',
            minio__service__pb2.MinIOHealthCheckRequest.SerializeToString,
            minio__service__pb2.MinIOHealthCheckResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
            _registered_method=True)



"""
gRPC Proto Generated Files
"""
from . import common_pb2
from . import minio_service_pb2, minio_service_pb2_grpc
from . import duckdb_service_pb2, duckdb_service_pb2_grpc
from . import mqtt_service_pb2, mqtt_service_pb2_grpc
from . import loki_service_pb2, loki_service_pb2_grpc
from . import redis_service_pb2, redis_service_pb2_grpc
from . import nats_service_pb2, nats_service_pb2_grpc
from . import postgres_service_pb2, postgres_service_pb2_grpc
from . import qdrant_service_pb2, qdrant_service_pb2_grpc
from . import neo4j_service_pb2, neo4j_service_pb2_grpc

__all__ = [
    'common_pb2',
    'minio_service_pb2', 'minio_service_pb2_grpc',
    'duckdb_service_pb2', 'duckdb_service_pb2_grpc',
    'mqtt_service_pb2', 'mqtt_service_pb2_grpc',
    'loki_service_pb2', 'loki_service_pb2_grpc',
    'redis_service_pb2', 'redis_service_pb2_grpc',
    'nats_service_pb2', 'nats_service_pb2_grpc',
    'postgres_service_pb2', 'postgres_service_pb2_grpc',
    'qdrant_service_pb2', 'qdrant_service_pb2_grpc',
    'neo4j_service_pb2', 'neo4j_service_pb2_grpc',
]



#!/usr/bin/env python3
"""
Async Redis gRPC Client
High-performance async Redis client using grpc.aio

Performance Benefits:
- True async I/O without GIL blocking
- Concurrent request execution (10x-50x throughput improvement)
- Automatic request batching support
- Memory-efficient connection pooling
"""

from typing import List, Dict, Optional, AsyncIterator, TYPE_CHECKING
from .async_base_client import AsyncBaseGRPCClient, BatchedRedisGet, BatchedRedisSet
from .proto import redis_service_pb2, redis_service_pb2_grpc

if TYPE_CHECKING:
    from .consul_client import ConsulRegistry


class AsyncRedisClient(AsyncBaseGRPCClient):
    """Async Redis gRPC Client for high-performance concurrent operations."""

    def __init__(
        self,
        host: Optional[str] = None,
        port: Optional[int] = None,
        user_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        lazy_connect: bool = True,
        enable_compression: bool = True,
        enable_retry: bool = True,
        consul_registry: Optional['ConsulRegistry'] = None,
        service_name_override: Optional[str] = None,
        enable_auto_batching: bool = False,
        auto_batch_size: int = 100,
        auto_batch_wait_ms: int = 5
    ):
        """
        Initialize async Redis client.

        Args:
            host: Service host (optional, will use Consul discovery if not provided)
            port: Service port (optional, will use Consul discovery if not provided)
            user_id: User ID
            organization_id: Organization ID
            lazy_connect: Lazy connection (default: True)
            enable_compression: Enable compression (default: True)
            enable_retry: Enable retry (default: True)
            consul_registry: ConsulRegistry instance for service discovery (optional)
            service_name_override: Override service name for Consul lookup (optional)
            enable_auto_batching: Enable automatic request batching (default: False)
            auto_batch_size: Max items per batch when auto-batching
            auto_batch_wait_ms: Max wait time before flush when auto-batching
        """
        super().__init__(
            host=host,
            port=port,
            user_id=user_id,
            lazy_connect=lazy_connect,
            enable_compression=enable_compression,
            enable_retry=enable_retry,
            consul_registry=consul_registry,
            service_name_override=service_name_override
        )
        self.organization_id = organization_id or 'default-org'

        # Auto-batching support
        self._enable_auto_batching = enable_auto_batching
        self._batched_get: Optional[BatchedRedisGet] = None
        self._batched_set: Optional[BatchedRedisSet] = None
        if enable_auto_batching:
            self._batched_get = BatchedRedisGet(self, auto_batch_size, auto_batch_wait_ms)
            self._batched_set = BatchedRedisSet(self, auto_batch_size, auto_batch_wait_ms)

    def _create_stub(self):
        """Create Redis service stub."""
        return redis_service_pb2_grpc.RedisServiceStub(self.channel)

    def service_name(self) -> str:
        return "Redis"

    def default_port(self) -> int:
        return 50055

    # ============================================
    # Health Check
    # ============================================

    async def health_check(self, deep_check: bool = False) -> Optional[Dict]:
        """Health check."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.RedisHealthCheckRequest(deep_check=deep_check)
            response = await self.stub.HealthCheck(request)

            return {
                'healthy': response.healthy,
                'redis_status': response.redis_status,
                'connected_clients': response.connected_clients,
                'used_memory_bytes': response.used_memory_bytes
            }

        except Exception as e:
            return self.handle_error(e, "Health check")

    # ============================================
    # String Operations
    # ============================================

    async def set(self, key: str, value: str, ttl_seconds: int = 0) -> Optional[bool]:
        """Set key-value."""
        try:
            await self._ensure_connected()

            if ttl_seconds > 0:
                from google.protobuf.duration_pb2 import Duration
                duration = Duration()
                duration.seconds = ttl_seconds

                request = redis_service_pb2.SetWithExpirationRequest(
                    user_id=self.user_id,
                    organization_id=self.organization_id,
                    key=key,
                    value=value,
                    expiration=duration
                )
                response = await self.stub.SetWithExpiration(request)
            else:
                request = redis_service_pb2.SetRequest(
                    user_id=self.user_id,
                    organization_id=self.organization_id,
                    key=key,
                    value=value
                )
                response = await self.stub.Set(request)

            return response.success

        except Exception as e:
            self.handle_error(e, "Set key-value")
            return False

    async def get(self, key: str) -> Optional[str]:
        """Get value by key."""
        # Use auto-batching if enabled
        if self._enable_auto_batching and self._batched_get:
            return await self._batched_get.get(key)

        try:
            await self._ensure_connected()
            request = redis_service_pb2.GetRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = await self.stub.Get(request)

            if response.found:
                return response.value
            return None

        except Exception as e:
            return self.handle_error(e, "Get key-value")

    async def delete(self, key: str) -> Optional[bool]:
        """Delete key."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.DeleteRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = await self.stub.Delete(request)
            return response.success

        except Exception as e:
            self.handle_error(e, "Delete key")
            return False

    async def exists(self, key: str) -> bool:
        """Check if key exists."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.ExistsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = await self.stub.Exists(request)
            return response.exists

        except Exception as e:
            self.handle_error(e, "Check key exists")
            return False

    async def set_with_ttl(self, key: str, value: str, ttl_seconds: int) -> Optional[bool]:
        """Set key-value with TTL."""
        return await self.set(key, value, ttl_seconds)

    async def append(self, key: str, value: str) -> Optional[int]:
        """Append value to key."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.AppendRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                value=value
            )

            response = await self.stub.Append(request)
            return response.length

        except Exception as e:
            return self.handle_error(e, "Append to key")

    # ============================================
    # Batch Operations
    # ============================================

    async def mset(self, key_values: Dict[str, str], ttl_seconds: int = 0) -> Optional[bool]:
        """Batch set key-values using ExecuteBatch for optimal performance."""
        try:
            await self._ensure_connected()

            batch_commands = []
            for key, value in key_values.items():
                cmd = redis_service_pb2.BatchCommand(
                    operation='SET',
                    key=key,
                    value=value
                )
                if ttl_seconds > 0:
                    from google.protobuf.duration_pb2 import Duration
                    expiration = Duration()
                    expiration.seconds = ttl_seconds
                    cmd.expiration.CopyFrom(expiration)
                batch_commands.append(cmd)

            request = redis_service_pb2.RedisBatchRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                commands=batch_commands
            )

            response = await self.stub.ExecuteBatch(request)

            if response.success and response.executed_count == len(key_values):
                return True
            return False

        except Exception as e:
            self.handle_error(e, "Batch set key-values")
            return False

    async def mget(self, keys: List[str]) -> Dict[str, str]:
        """Batch get key-values."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.GetMultipleRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                keys=keys
            )

            response = await self.stub.GetMultiple(request)

            values = {kv.key: kv.value for kv in response.values}
            return values

        except Exception as e:
            return self.handle_error(e, "Batch get key-values") or {}

    async def delete_multiple(self, keys: List[str]) -> Optional[int]:
        """Delete multiple keys."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.DeleteMultipleRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                keys=keys
            )

            response = await self.stub.DeleteMultiple(request)

            if response.success:
                return response.deleted_count
            return 0

        except Exception as e:
            self.handle_error(e, "Delete multiple keys")
            return 0

    async def execute_batch(self, commands: List[Dict]) -> Optional[Dict]:
        """Execute batch commands."""
        try:
            await self._ensure_connected()

            batch_commands = []
            for cmd in commands:
                from google.protobuf.duration_pb2 import Duration

                expiration = None
                if 'expiration' in cmd and cmd['expiration']:
                    expiration = Duration()
                    expiration.seconds = cmd['expiration']

                batch_cmd = redis_service_pb2.BatchCommand(
                    operation=cmd.get('operation', ''),
                    key=cmd.get('key', ''),
                    value=cmd.get('value', ''),
                    expiration=expiration
                )
                batch_commands.append(batch_cmd)

            request = redis_service_pb2.RedisBatchRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                commands=batch_commands
            )

            response = await self.stub.ExecuteBatch(request)

            return {
                'success': response.success,
                'executed_count': response.executed_count,
                'errors': list(response.errors)
            }

        except Exception as e:
            self.handle_error(e, "Execute batch")
            return None

    # ============================================
    # Counter Operations
    # ============================================

    async def incr(self, key: str, delta: int = 1) -> Optional[int]:
        """Increment."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.IncrementRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                delta=delta
            )

            response = await self.stub.Increment(request)
            return response.value

        except Exception as e:
            return self.handle_error(e, "Increment")

    async def decr(self, key: str, delta: int = 1) -> Optional[int]:
        """Decrement."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.DecrementRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                delta=delta
            )

            response = await self.stub.Decrement(request)
            return response.value

        except Exception as e:
            return self.handle_error(e, "Decrement")

    # ============================================
    # Key Operations
    # ============================================

    async def expire(self, key: str, seconds: int) -> Optional[bool]:
        """Set expiration time."""
        try:
            await self._ensure_connected()

            from google.protobuf.duration_pb2 import Duration
            duration = Duration()
            duration.seconds = seconds

            request = redis_service_pb2.ExpireRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                expiration=duration
            )

            response = await self.stub.Expire(request)
            return response.success

        except Exception as e:
            self.handle_error(e, "Set expiration")
            return False

    async def ttl(self, key: str) -> Optional[int]:
        """Get time to live (seconds)."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.GetTTLRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = await self.stub.GetTTL(request)
            return response.ttl_seconds

        except Exception as e:
            return self.handle_error(e, "Get TTL")

    async def rename(self, old_key: str, new_key: str) -> Optional[bool]:
        """Rename key."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.RenameRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                old_key=old_key,
                new_key=new_key
            )

            response = await self.stub.Rename(request)
            return response.success

        except Exception as e:
            self.handle_error(e, "Rename key")
            return False

    async def list_keys(self, pattern: str = "*", limit: int = 100) -> List[str]:
        """List keys matching pattern."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.ListKeysRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                pattern=pattern,
                limit=limit
            )

            response = await self.stub.ListKeys(request)
            return list(response.keys)

        except Exception as e:
            return self.handle_error(e, "List keys") or []

    # ============================================
    # List Operations
    # ============================================

    async def lpush(self, key: str, values: List[str]) -> Optional[int]:
        """Left push to list."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.LPushRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                values=values
            )

            response = await self.stub.LPush(request)
            return response.length

        except Exception as e:
            return self.handle_error(e, "Left push to list")

    async def rpush(self, key: str, values: List[str]) -> Optional[int]:
        """Right push to list."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.RPushRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                values=values
            )

            response = await self.stub.RPush(request)
            return response.length

        except Exception as e:
            return self.handle_error(e, "Right push to list")

    async def lrange(self, key: str, start: int = 0, stop: int = -1) -> List[str]:
        """Get list range."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.LRangeRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                start=start,
                stop=stop
            )

            response = await self.stub.LRange(request)
            return list(response.values)

        except Exception as e:
            return self.handle_error(e, "Get list range") or []

    async def lpop(self, key: str) -> Optional[str]:
        """Pop element from left of list."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.LPopRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = await self.stub.LPop(request)

            if response.found:
                return response.value
            return None

        except Exception as e:
            return self.handle_error(e, "Left pop from list")

    async def rpop(self, key: str) -> Optional[str]:
        """Pop element from right of list."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.RPopRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = await self.stub.RPop(request)

            if response.found:
                return response.value
            return None

        except Exception as e:
            return self.handle_error(e, "Right pop from list")

    async def llen(self, key: str) -> Optional[int]:
        """Get list length."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.LLenRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = await self.stub.LLen(request)
            return response.length

        except Exception as e:
            return self.handle_error(e, "Get list length")

    # ============================================
    # Hash Operations
    # ============================================

    async def hset(self, key: str, field: str, value: str) -> Optional[bool]:
        """Set hash field."""
        try:
            await self._ensure_connected()

            hash_field = redis_service_pb2.HashField(field=field, value=value)

            request = redis_service_pb2.HSetRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                fields=[hash_field]
            )

            response = await self.stub.HSet(request)
            return response.success

        except Exception as e:
            self.handle_error(e, "Set hash field")
            return False

    async def hget(self, key: str, field: str) -> Optional[str]:
        """Get hash field."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.HGetRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                field=field
            )

            response = await self.stub.HGet(request)

            if response.found:
                return response.value
            return None

        except Exception as e:
            return self.handle_error(e, "Get hash field")

    async def hgetall(self, key: str) -> Dict[str, str]:
        """Get all hash fields."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.HGetAllRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = await self.stub.HGetAll(request)

            fields = {f.field: f.value for f in response.fields}
            return fields

        except Exception as e:
            return self.handle_error(e, "Get all hash fields") or {}

    async def hdelete(self, key: str, fields: List[str]) -> Optional[int]:
        """Delete hash fields."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.HDeleteRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                fields=fields
            )

            response = await self.stub.HDelete(request)

            if response.success:
                return response.deleted_count
            return 0

        except Exception as e:
            self.handle_error(e, "Delete hash fields")
            return 0

    async def hexists(self, key: str, field: str) -> bool:
        """Check if hash field exists."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.HExistsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                field=field
            )

            response = await self.stub.HExists(request)
            return response.exists

        except Exception as e:
            self.handle_error(e, "Check hash field exists")
            return False

    # ============================================
    # Set Operations
    # ============================================

    async def sadd(self, key: str, members: List[str]) -> Optional[int]:
        """Add members to set."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.SAddRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                members=members
            )

            response = await self.stub.SAdd(request)
            return response.added_count

        except Exception as e:
            return self.handle_error(e, "Add to set")

    async def sremove(self, key: str, members: List[str]) -> Optional[int]:
        """Remove members from set."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.SRemoveRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                members=members
            )

            response = await self.stub.SRemove(request)
            return response.removed_count

        except Exception as e:
            return self.handle_error(e, "Remove from set")

    async def smembers(self, key: str) -> List[str]:
        """Get all set members."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.SMembersRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = await self.stub.SMembers(request)
            return list(response.members)

        except Exception as e:
            return self.handle_error(e, "Get set members") or []

    async def sismember(self, key: str, member: str) -> bool:
        """Check if member is in set."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.SIsMemberRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                member=member
            )

            response = await self.stub.SIsMember(request)
            return response.is_member

        except Exception as e:
            self.handle_error(e, "Check set membership")
            return False

    async def scard(self, key: str) -> Optional[int]:
        """Get set cardinality (size)."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.SCardRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = await self.stub.SCard(request)
            return response.count

        except Exception as e:
            return self.handle_error(e, "Get set cardinality")

    # ============================================
    # Sorted Set Operations
    # ============================================

    async def zadd(self, key: str, score_members: Dict[str, float]) -> Optional[int]:
        """Add members to sorted set with scores."""
        try:
            await self._ensure_connected()

            members = [
                redis_service_pb2.ZSetMember(member=member, score=score)
                for member, score in score_members.items()
            ]

            request = redis_service_pb2.ZAddRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                members=members
            )

            response = await self.stub.ZAdd(request)
            return response.added_count

        except Exception as e:
            self.handle_error(e, "Add to sorted set")
            return None

    async def zrange(self, key: str, start: int = 0, stop: int = -1, with_scores: bool = False) -> List:
        """Get sorted set range."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.ZRangeRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                start=start,
                stop=stop,
                with_scores=with_scores
            )

            response = await self.stub.ZRange(request)

            if with_scores:
                return [(m.member, m.score) for m in response.members]
            return [m.member for m in response.members]

        except Exception as e:
            return self.handle_error(e, "Get sorted set range") or []

    async def zrem(self, key: str, members: List[str]) -> Optional[int]:
        """Remove members from sorted set."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.ZRemoveRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                members=members
            )

            response = await self.stub.ZRemove(request)
            return response.removed_count

        except Exception as e:
            self.handle_error(e, "Remove from sorted set")
            return None

    async def zrank(self, key: str, member: str) -> Optional[int]:
        """Get member rank in sorted set."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.ZRankRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                member=member
            )

            response = await self.stub.ZRank(request)

            if response.found:
                return response.rank
            return None

        except Exception as e:
            return self.handle_error(e, "Get sorted set rank")

    async def zscore(self, key: str, member: str) -> Optional[float]:
        """Get member score in sorted set."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.ZScoreRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key,
                member=member
            )

            response = await self.stub.ZScore(request)

            if response.found:
                return response.score
            return None

        except Exception as e:
            return self.handle_error(e, "Get sorted set score")

    async def zcard(self, key: str) -> Optional[int]:
        """Get sorted set cardinality (size)."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.ZCardRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                key=key
            )

            response = await self.stub.ZCard(request)
            return response.count

        except Exception as e:
            return self.handle_error(e, "Get sorted set cardinality")

    # ============================================
    # Distributed Lock Operations
    # ============================================

    async def acquire_lock(self, lock_key: str, ttl_seconds: int = 10, wait_timeout_seconds: int = 5) -> Optional[str]:
        """Acquire distributed lock."""
        try:
            await self._ensure_connected()

            from google.protobuf.duration_pb2 import Duration
            ttl = Duration()
            ttl.seconds = ttl_seconds
            wait_timeout = Duration()
            wait_timeout.seconds = wait_timeout_seconds

            request = redis_service_pb2.AcquireLockRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                lock_key=lock_key,
                ttl=ttl,
                wait_timeout=wait_timeout
            )

            response = await self.stub.AcquireLock(request)

            if response.acquired:
                return response.lock_id
            return None

        except Exception as e:
            self.handle_error(e, "Acquire lock")
            return None

    async def release_lock(self, lock_key: str, lock_id: str) -> bool:
        """Release distributed lock."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.ReleaseLockRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                lock_key=lock_key,
                lock_id=lock_id
            )

            response = await self.stub.ReleaseLock(request)
            return response.released

        except Exception as e:
            self.handle_error(e, "Release lock")
            return False

    async def renew_lock(self, lock_key: str, lock_id: str, ttl_seconds: int = 10) -> bool:
        """Renew distributed lock."""
        try:
            await self._ensure_connected()

            from google.protobuf.duration_pb2 import Duration
            ttl = Duration()
            ttl.seconds = ttl_seconds

            request = redis_service_pb2.RenewLockRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                lock_key=lock_key,
                lock_id=lock_id,
                ttl=ttl
            )

            response = await self.stub.RenewLock(request)
            return response.renewed

        except Exception as e:
            self.handle_error(e, "Renew lock")
            return False

    # ============================================
    # Pub/Sub Operations
    # ============================================

    async def publish(self, channel: str, message: str) -> Optional[int]:
        """Publish message to channel."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.PublishRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                channel=channel,
                message=message
            )

            response = await self.stub.Publish(request)
            return response.subscriber_count

        except Exception as e:
            return self.handle_error(e, "Publish message")

    async def subscribe(self, channels: List[str]) -> AsyncIterator[Dict]:
        """
        Subscribe to channels (streaming).

        Yields:
            Dict with 'channel', 'message', 'timestamp' for each message
        """
        try:
            await self._ensure_connected()
            request = redis_service_pb2.SubscribeRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                channels=channels
            )

            async for message in self.stub.Subscribe(request):
                yield {
                    'channel': message.channel,
                    'message': message.message,
                    'timestamp': message.timestamp
                }

        except Exception as e:
            self.handle_error(e, "Subscribe to channels")

    # ============================================
    # Session Management
    # ============================================

    async def create_session(self, data: Dict[str, str], ttl_seconds: int = 3600) -> Optional[str]:
        """Create session."""
        try:
            await self._ensure_connected()

            from google.protobuf.duration_pb2 import Duration
            ttl = Duration()
            ttl.seconds = ttl_seconds

            request = redis_service_pb2.CreateSessionRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                data=data,
                ttl=ttl
            )

            response = await self.stub.CreateSession(request)
            return response.session_id

        except Exception as e:
            return self.handle_error(e, "Create session")

    async def get_session(self, session_id: str) -> Optional[Dict]:
        """Get session."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.GetSessionRequest(
                user_id=self.user_id,
                session_id=session_id
            )

            response = await self.stub.GetSession(request)

            if response.found:
                return self._proto_map_to_dict(response.session.data)
            return None

        except Exception as e:
            return self.handle_error(e, "Get session")

    async def delete_session(self, session_id: str) -> bool:
        """Delete session."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.DeleteSessionRequest(
                user_id=self.user_id,
                session_id=session_id
            )

            response = await self.stub.DeleteSession(request)
            return response.success

        except Exception as e:
            self.handle_error(e, "Delete session")
            return False

    # ============================================
    # Statistics
    # ============================================

    async def get_statistics(self) -> Optional[Dict]:
        """Get Redis statistics."""
        try:
            await self._ensure_connected()
            request = redis_service_pb2.GetStatisticsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id
            )

            response = await self.stub.GetStatistics(request)

            return {
                'total_keys': response.total_keys,
                'memory_used_bytes': response.memory_used_bytes,
                'commands_processed': response.commands_processed,
                'connections_received': response.connections_received,
                'hit_rate': response.hit_rate,
                'key_type_distribution': self._proto_map_to_dict(response.key_type_distribution)
            }

        except Exception as e:
            return self.handle_error(e, "Get statistics")

    # ============================================
    # Concurrent Operations Helper
    # ============================================

    async def get_many_concurrent(self, keys: List[str]) -> Dict[str, Optional[str]]:
        """
        Get multiple keys concurrently using asyncio.gather.

        This is useful when you want individual get calls to run concurrently
        without using the batch mget operation.

        Args:
            keys: List of keys to get

        Returns:
            Dict mapping keys to their values (None if not found)
        """
        import asyncio

        async def get_single(key: str) -> tuple:
            value = await self.get(key)
            return (key, value)

        results = await asyncio.gather(*[get_single(k) for k in keys])
        return dict(results)

    async def set_many_concurrent(self, key_values: Dict[str, str], ttl_seconds: int = 0) -> Dict[str, bool]:
        """
        Set multiple key-values concurrently using asyncio.gather.

        Args:
            key_values: Dict of key-value pairs
            ttl_seconds: TTL for all keys

        Returns:
            Dict mapping keys to success status
        """
        import asyncio

        async def set_single(key: str, value: str) -> tuple:
            success = await self.set(key, value, ttl_seconds)
            return (key, success)

        results = await asyncio.gather(*[set_single(k, v) for k, v in key_values.items()])
        return dict(results)


# Example usage
if __name__ == '__main__':
    import asyncio

    async def main():
        async with AsyncRedisClient(host='localhost', port=50055, user_id='test_user') as client:
            # Health check
            health = await client.health_check()
            print(f"Health: {health}")

            # Basic operations
            await client.set('async:key1', 'value1')
            value = await client.get('async:key1')
            print(f"Got: {value}")

            # Batch operations
            await client.mset({
                'async:key2': 'value2',
                'async:key3': 'value3',
                'async:key4': 'value4'
            })

            values = await client.mget(['async:key2', 'async:key3', 'async:key4'])
            print(f"Batch get: {values}")

            # Concurrent operations
            concurrent_results = await client.get_many_concurrent([
                'async:key1', 'async:key2', 'async:key3'
            ])
            print(f"Concurrent get: {concurrent_results}")

    asyncio.run(main())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Base Event Publisher

Generic event publisher that can be extended for business-specific needs.
"""

import json
import logging
from typing import Optional, Dict, Any, TYPE_CHECKING
from abc import ABC, abstractmethod

from ..nats_client import NATSClient
from .base_event_models import BaseEvent

if TYPE_CHECKING:
    from ..consul_client import ConsulRegistry

logger = logging.getLogger(__name__)


class BaseEventPublisher(ABC):
    """
    Base class for event publishers.

    Extend this for your service-specific publishers.

    Example:
        class UserEventPublisher(BaseEventPublisher):
            def service_name(self) -> str:
                return "user_service"

            async def publish_user_created(self, user_id: str, email: str) -> bool:
                event = UserCreatedEvent(
                    event_type="user.created",
                    user_id=user_id,
                    email=email
                )
                return await self.publish_event(event, subject="user.created")
    """

    def __init__(
        self,
        nats_host: Optional[str] = None,
        nats_port: Optional[int] = None,
        user_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        enable_compression: bool = False,
        service_name: Optional[str] = None,
        consul_registry: Optional['ConsulRegistry'] = None
    ):
        """
        Initialize event publisher.

        Args:
            nats_host: NATS service host (optional, will use Consul discovery if not provided)
            nats_port: NATS service gRPC port (optional, will use Consul discovery if not provided)
            user_id: Default user ID for events
            organization_id: Default organization ID
            enable_compression: Enable message compression
            service_name: Name of the service publishing events
            consul_registry: ConsulRegistry instance for service discovery (optional)
        """
        self.nats_client = NATSClient(
            host=nats_host,
            port=nats_port,
            user_id=user_id or service_name,
            organization_id=organization_id,
            enable_compression=enable_compression,
            consul_registry=consul_registry
        )
        self.default_user_id = user_id
        self.default_org_id = organization_id
        self._service_name = service_name
        self.consul_registry = consul_registry

    @abstractmethod
    def service_name(self) -> str:
        """
        Return the name of the service publishing events.

        This is used for logging and tracking event sources.
        """
        pass

    def __enter__(self):
        """Context manager entry"""
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Context manager exit"""
        self.close()

    async def __aenter__(self):
        """Async context manager entry"""
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """Async context manager exit"""
        await self.aclose()

    def close(self):
        """Close NATS connection"""
        if hasattr(self.nats_client, 'close'):
            self.nats_client.close()

    async def aclose(self):
        """Async close NATS connection"""
        if hasattr(self.nats_client, 'aclose'):
            await self.nats_client.aclose()
        elif hasattr(self.nats_client, 'close'):
            self.nats_client.close()

    def _serialize_event(self, event: BaseEvent) -> bytes:
        """
        Serialize Pydantic event to JSON bytes.

        Args:
            event: Pydantic event model

        Returns:
            JSON bytes ready for NATS
        """
        # Set source service in metadata if not already set
        if hasattr(event, 'metadata') and event.metadata.source_service is None:
            event.metadata.source_service = self.service_name()

        # Use Pydantic's JSON serialization with custom encoders
        json_str = event.model_dump_json()
        return json_str.encode('utf-8')

    async def publish_event(
        self,
        event: BaseEvent,
        subject: str,
        headers: Optional[Dict[str, str]] = None
    ) -> bool:
        """
        Publish an event to NATS.

        Args:
            event: Event to publish (must extend BaseEvent)
            subject: NATS subject
            headers: Optional headers

        Returns:
            True if published successfully, False otherwise
        """
        try:
            # Serialize event
            data = self._serialize_event(event)

            # Add event type to headers
            if headers is None:
                headers = {}
            headers['event_type'] = event.event_type

            # Publish to NATS
            result = self.nats_client.publish(
                subject=subject,
                data=data,
                headers=headers
            )

            if result and result.get('success'):
                logger.info(
                    f"[{self.service_name()}] Published event: {event.event_type} "
                    f"to subject: {subject}"
                )
                return True
            else:
                logger.error(
                    f"[{self.service_name()}] Failed to publish event: {result}"
                )
                return False

        except Exception as e:
            logger.error(
                f"[{self.service_name()}] Error publishing event {event.event_type}: {e}",
                exc_info=True
            )
            return False

    async def publish_raw(
        self,
        subject: str,
        data: bytes,
        headers: Optional[Dict[str, str]] = None
    ) -> bool:
        """
        Publish raw data to NATS (for advanced use cases).

        Args:
            subject: NATS subject
            data: Raw bytes to publish
            headers: Optional headers

        Returns:
            True if published successfully
        """
        try:
            result = self.nats_client.publish(
                subject=subject,
                data=data,
                headers=headers
            )

            if result and result.get('success'):
                logger.info(f"[{self.service_name()}] Published to subject: {subject}")
                return True
            else:
                logger.error(f"[{self.service_name()}] Failed to publish: {result}")
                return False

        except Exception as e:
            logger.error(
                f"[{self.service_name()}] Error publishing to {subject}: {e}",
                exc_info=True
            )
            return False



#!/usr/bin/env python3
"""
Loki gRPC Client
Loki log aggregation client
"""

from typing import List, Dict, Optional, TYPE_CHECKING
from datetime import datetime
from .base_client import BaseGRPCClient
from .proto import loki_service_pb2, loki_service_pb2_grpc
from google.protobuf.timestamp_pb2 import Timestamp

if TYPE_CHECKING:
    from .consul_client import ConsulRegistry


class LokiClient(BaseGRPCClient):
    """Loki gRPC Client"""

    def __init__(self, host: Optional[str] = None, port: Optional[int] = None, user_id: Optional[str] = None,
                 organization_id: Optional[str] = None, lazy_connect: bool = True,
                 enable_compression: bool = True, enable_retry: bool = True,
                 consul_registry: Optional['ConsulRegistry'] = None, service_name_override: Optional[str] = None):
        """
        Initialize Loki client

        Args:
            host: Service host (optional, will use Consul discovery if not provided)
            port: Service port (optional, will use Consul discovery if not provided)
            user_id: User ID
            organization_id: Organization ID
            lazy_connect: Lazy connection (default: True)
            enable_compression: Enable compression (default: True)
            enable_retry: Enable retry (default: True)
            consul_registry: ConsulRegistry instance for service discovery (optional)
            service_name_override: Override service name for Consul lookup (optional, defaults to 'loki')
        """
        super().__init__(
            host=host,
            port=port,
            user_id=user_id,
            lazy_connect=lazy_connect,
            enable_compression=enable_compression,
            enable_retry=enable_retry,
            consul_registry=consul_registry,
            service_name_override=service_name_override
        )
        self.organization_id = organization_id or 'default-org'

    def _create_stub(self):
        """Create Loki service stub"""
        return loki_service_pb2_grpc.LokiServiceStub(self.channel)

    def service_name(self) -> str:
        return "Loki"

    def default_port(self) -> int:
        return 50054

    def health_check(self) -> Optional[Dict]:
        """Health check"""
        try:
            self._ensure_connected()
            request = loki_service_pb2.LokiHealthCheckRequest(
                deep_check=False
            )
            response = self.stub.HealthCheck(request)

            print(f"✅ [Loki] Service healthy: {response.healthy}")
            print(f"   Loki status: {response.loki_status}")
            print(f"   Can write: {response.can_write}, Can read: {response.can_read}")

            return {
                'healthy': response.healthy,
                'loki_status': response.loki_status,
                'can_write': response.can_write,
                'can_read': response.can_read
            }

        except Exception as e:
            return self.handle_error(e, "Health check")

    def push_log(self, message: str, labels: Dict[str, str] = None,
                 timestamp: Optional[datetime] = None) -> Optional[Dict]:
        """Push single log entry"""
        try:
            self._ensure_connected()

            # Prepare timestamp
            ts = Timestamp()
            if timestamp:
                ts.FromDatetime(timestamp)
            else:
                ts.GetCurrentTime()

            # Prepare labels
            log_labels = labels or {}

            # Create log entry
            entry = loki_service_pb2.LogEntry(
                timestamp=ts,
                line=message,
                labels=log_labels
            )

            request = loki_service_pb2.PushLogRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                entry=entry
            )

            response = self.stub.PushLog(request)

            if response.success:
                print(f"✅ [Loki] Log pushed successfully")
                return {'success': True}
            else:
                print(f"⚠️  [Loki] Log push failed")
                return None

        except Exception as e:
            return self.handle_error(e, "Push log")

    def push_simple_log(self, message: str, service: str, level: str = 'INFO',
                       extra_labels: Dict[str, str] = None,
                       timestamp: Optional[datetime] = None) -> Optional[Dict]:
        """Push simple log (auto-add common labels)"""
        try:
            self._ensure_connected()

            # Prepare timestamp
            ts = Timestamp()
            if timestamp:
                ts.FromDatetime(timestamp)
            else:
                ts.GetCurrentTime()

            # Map log level
            level_map = {
                'DEBUG': loki_service_pb2.LOG_LEVEL_DEBUG,
                'INFO': loki_service_pb2.LOG_LEVEL_INFO,
                'WARNING': loki_service_pb2.LOG_LEVEL_WARN,
                'WARN': loki_service_pb2.LOG_LEVEL_WARN,
                'ERROR': loki_service_pb2.LOG_LEVEL_ERROR,
                'CRITICAL': loki_service_pb2.LOG_LEVEL_FATAL,
                'FATAL': loki_service_pb2.LOG_LEVEL_FATAL
            }
            log_level = level_map.get(level.upper(), loki_service_pb2.LOG_LEVEL_INFO)

            request = loki_service_pb2.PushSimpleLogRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                timestamp=ts,
                service=service,
                level=log_level,
                message=message,
                extra_labels=extra_labels or {}
            )

            response = self.stub.PushSimpleLog(request)

            if response.success:
                print(f"✅ [Loki] Log pushed: [{level}] {message[:50]}...")
                return {'success': True}
            else:
                print(f"⚠️  [Loki] Log push failed")
                return None

        except Exception as e:
            return self.handle_error(e, "Push simple log")

    def push_log_batch(self, entries: List[Dict]) -> Optional[Dict]:
        """Push batch logs"""
        try:
            self._ensure_connected()

            log_entries = []
            for entry in entries:
                ts = Timestamp()
                if 'timestamp' in entry and entry['timestamp']:
                    ts.FromDatetime(entry['timestamp'])
                else:
                    ts.GetCurrentTime()

                log_entry = loki_service_pb2.LogEntry(
                    timestamp=ts,
                    line=entry.get('message', ''),
                    labels=entry.get('labels', {})
                )
                log_entries.append(log_entry)

            request = loki_service_pb2.PushLogBatchRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                entries=log_entries
            )

            response = self.stub.PushLogBatch(request)

            print(f"✅ [Loki] Batch push completed: {response.accepted_count} accepted, {response.rejected_count} rejected")

            return {
                'success': response.success,
                'accepted_count': response.accepted_count,
                'rejected_count': response.rejected_count,
                'errors': list(response.errors) if response.errors else []
            }

        except Exception as e:
            return self.handle_error(e, "Push log batch")

    def query_logs(self, query: str, limit: int = 100,
                   start_time: Optional[datetime] = None,
                   end_time: Optional[datetime] = None) -> List[Dict]:
        """Query logs"""
        try:
            self._ensure_connected()

            # Prepare time range
            start_ts = Timestamp()
            end_ts = Timestamp()

            if start_time:
                start_ts.FromDatetime(start_time)
            if end_time:
                end_ts.FromDatetime(end_time)
            else:
                end_ts.GetCurrentTime()

            request = loki_service_pb2.QueryLogsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                query=query,
                limit=limit,
                start=start_ts if start_time else None,
                end=end_ts
            )

            response = self.stub.QueryLogs(request)

            logs = []
            for entry in response.entries:
                logs.append({
                    'timestamp': entry.timestamp.ToDatetime(),
                    'message': entry.line,
                    'labels': dict(entry.labels)
                })

            print(f"✅ [Loki] Found {len(logs)} logs (total: {response.total_count})")
            return logs

        except Exception as e:
            return self.handle_error(e, "Query logs") or []

    def get_labels(self) -> List[str]:
        """Get available labels"""
        try:
            self._ensure_connected()
            request = loki_service_pb2.GetLabelsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id
            )

            response = self.stub.GetLabels(request)

            print(f"✅ [Loki] Found {len(response.labels)} labels")
            return list(response.labels)

        except Exception as e:
            return self.handle_error(e, "Get labels") or []

    def get_label_values(self, label: str) -> List[str]:
        """Get values for specific label"""
        try:
            self._ensure_connected()
            request = loki_service_pb2.GetLabelValuesRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                label_name=label
            )

            response = self.stub.GetLabelValues(request)

            print(f"✅ [Loki] Label '{label}' has {len(response.values)} values")
            return list(response.values)

        except Exception as e:
            return self.handle_error(e, f"Get label values ({label})") or []

    def push_log_stream(self, log_generator):
        """
        Push logs via streaming (high throughput)
        
        Args:
            log_generator: Generator yielding dicts with 'message', 'labels', 'timestamp'
        """
        try:
            self._ensure_connected()
            
            def request_generator():
                for log_data in log_generator:
                    ts = Timestamp()
                    if 'timestamp' in log_data and log_data['timestamp']:
                        ts.FromDatetime(log_data['timestamp'])
                    else:
                        ts.GetCurrentTime()
                    
                    entry = loki_service_pb2.LogEntry(
                        timestamp=ts,
                        line=log_data.get('message', ''),
                        labels=log_data.get('labels', {})
                    )
                    
                    yield loki_service_pb2.PushLogRequest(
                        user_id=self.user_id,
                        organization_id=self.organization_id,
                        entry=entry
                    )
            
            response = self.stub.PushLogStream(request_generator())
            
            print(f"✅ [Loki] Stream push completed: {response.accepted_count} accepted, {response.rejected_count} rejected")
            
            return {
                'success': response.success,
                'accepted_count': response.accepted_count,
                'rejected_count': response.rejected_count
            }

        except Exception as e:
            return self.handle_error(e, "Push log stream")

    def query_range(self, query: str, start_time: datetime, end_time: datetime,
                    step: int = 60, limit: int = 1000) -> Optional[Dict]:
        """
        Range query with time series aggregation
        
        Args:
            query: LogQL query
            start_time: Start time
            end_time: End time
            step: Step size in seconds
            limit: Max results
        """
        try:
            self._ensure_connected()
            
            start_ts = Timestamp()
            start_ts.FromDatetime(start_time)
            end_ts = Timestamp()
            end_ts.FromDatetime(end_time)
            
            request = loki_service_pb2.QueryRangeRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                query=query,
                start=start_ts,
                end=end_ts,
                step=step,
                limit=limit
            )

            response = self.stub.QueryRange(request)

            results = {
                'result_type': response.result_type,
                'matrix_results': [],
                'stream_results': [],
                'stats': dict(response.stats)
            }
            
            # Parse matrix results
            for matrix in response.matrix_results:
                results['matrix_results'].append({
                    'metric': dict(matrix.metric),
                    'values': [(p.timestamp, p.value) for p in matrix.values]
                })
            
            # Parse stream results
            for stream in response.stream_results:
                results['stream_results'].append({
                    'stream': dict(stream.stream),
                    'entries': [{
                        'timestamp': e.timestamp,
                        'line': e.line,
                        'labels': dict(e.labels)
                    } for e in stream.entries]
                })
            
            print(f"✅ [Loki] Range query completed: {len(results['stream_results'])} streams")
            return results

        except Exception as e:
            return self.handle_error(e, "Query range")

    def tail_logs(self, query: str, callback=None, limit: int = 100, delay_for: int = 0):
        """
        Tail logs in real-time (like tail -f)
        
        Args:
            query: LogQL query
            callback: Function to call for each log entry
            limit: Max entries to tail
            delay_for: Delay seconds before starting
        """
        try:
            self._ensure_connected()
            
            start_ts = None
            if delay_for > 0:
                start_ts = Timestamp()
                start_ts.GetCurrentTime()
            
            request = loki_service_pb2.TailLogsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                query=query,
                limit=limit,
                delay_for=delay_for,
                start=start_ts
            )

            print(f"✅ [Loki] Tailing logs: {query}")
            
            for entry in self.stub.TailLogs(request):
                if callback:
                    callback(entry.timestamp, entry.line, dict(entry.labels))
                else:
                    print(f"📝 [{entry.timestamp}] {entry.line}")

        except Exception as e:
            self.handle_error(e, "Tail logs")

    def query_stats(self, query: str, start_time: datetime, end_time: datetime) -> Optional[Dict]:
        """Get query statistics"""
        try:
            self._ensure_connected()
            
            start_ts = Timestamp()
            start_ts.FromDatetime(start_time)
            end_ts = Timestamp()
            end_ts.FromDatetime(end_time)
            
            request = loki_service_pb2.QueryStatsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                query=query,
                start=start_ts,
                end=end_ts
            )

            response = self.stub.QueryStats(request)

            stats = {
                'total_entries': response.total_entries,
                'total_bytes': response.total_bytes,
                'streams_count': response.streams_count,
                'query_time_ms': response.query_time_ms,
                'level_distribution': dict(response.level_distribution),
                'top_labels': list(response.top_labels)
            }
            
            print(f"✅ [Loki] Query stats: {response.total_entries} entries, {response.streams_count} streams")
            print(f"   Query time: {response.query_time_ms:.2f}ms")
            return stats

        except Exception as e:
            return self.handle_error(e, "Query stats")

    def validate_labels(self, labels: Dict[str, str]) -> Optional[Dict]:
        """Validate labels format"""
        try:
            self._ensure_connected()
            request = loki_service_pb2.ValidateLabelsRequest(
                labels=labels
            )

            response = self.stub.ValidateLabels(request)

            if response.valid:
                print(f"✅ [Loki] Labels valid")
            else:
                print(f"⚠️  [Loki] Labels invalid: {response.message}")
                print(f"   Violations: {', '.join(response.violations)}")
            
            return {
                'valid': response.valid,
                'violations': list(response.violations),
                'message': response.message
            }

        except Exception as e:
            return self.handle_error(e, "Validate labels")

    def list_streams(self, label_filter: Dict[str, str] = None, page: int = 1, 
                     page_size: int = 50) -> Optional[Dict]:
        """List log streams"""
        try:
            self._ensure_connected()
            request = loki_service_pb2.ListStreamsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                label_filter=label_filter or {},
                page=page,
                page_size=page_size
            )

            response = self.stub.ListStreams(request)

            streams = []
            for stream in response.streams:
                streams.append({
                    'stream_id': stream.stream_id,
                    'labels': dict(stream.labels),
                    'entry_count': stream.entry_count,
                    'bytes': stream.bytes,
                    'first_entry_at': stream.first_entry_at,
                    'last_entry_at': stream.last_entry_at
                })
            
            print(f"✅ [Loki] Listed {len(streams)} streams (total: {response.total_count})")
            return {
                'streams': streams,
                'total_count': response.total_count,
                'page': response.page,
                'page_size': response.page_size
            }

        except Exception as e:
            return self.handle_error(e, "List streams")

    def get_stream_info(self, stream_id: str) -> Optional[Dict]:
        """Get stream information"""
        try:
            self._ensure_connected()
            request = loki_service_pb2.GetStreamInfoRequest(
                user_id=self.user_id,
                stream_id=stream_id
            )

            response = self.stub.GetStreamInfo(request)

            stream = response.stream
            info = {
                'stream_id': stream.stream_id,
                'labels': dict(stream.labels),
                'entry_count': stream.entry_count,
                'bytes': stream.bytes,
                'first_entry_at': stream.first_entry_at,
                'last_entry_at': stream.last_entry_at,
                'user_id': stream.user_id,
                'organization_id': stream.organization_id
            }
            
            print(f"✅ [Loki] Stream info: {stream_id}")
            print(f"   Entries: {stream.entry_count}, Bytes: {stream.bytes}")
            return info

        except Exception as e:
            return self.handle_error(e, "Get stream info")

    def delete_stream(self, stream_id: str, start_time: datetime = None, 
                     end_time: datetime = None) -> Optional[Dict]:
        """Delete log stream"""
        try:
            self._ensure_connected()
            
            start_ts = None
            end_ts = None
            if start_time:
                start_ts = Timestamp()
                start_ts.FromDatetime(start_time)
            if end_time:
                end_ts = Timestamp()
                end_ts.FromDatetime(end_time)
            
            request = loki_service_pb2.DeleteStreamRequest(
                user_id=self.user_id,
                stream_id=stream_id,
                start=start_ts,
                end=end_ts
            )

            response = self.stub.DeleteStream(request)

            if response.success:
                print(f"✅ [Loki] Stream deleted: {stream_id}")
                print(f"   Deleted {response.deleted_entries} entries")
                return {
                    'success': True,
                    'deleted_entries': response.deleted_entries,
                    'message': response.message
                }
            else:
                print(f"⚠️  [Loki] Stream deletion failed: {response.message}")
                return None

        except Exception as e:
            return self.handle_error(e, "Delete stream")

    def export_logs(self, query: str, start_time: datetime, end_time: datetime,
                    format: str = 'JSONL', chunk_size: int = 1000, callback=None):
        """
        Export logs to file (streaming)
        
        Args:
            query: LogQL query
            start_time: Start time
            end_time: End time
            format: Export format (JSON, JSONL, CSV, TEXT)
            chunk_size: Chunk size
            callback: Function to call for each chunk
        """
        try:
            self._ensure_connected()
            
            start_ts = Timestamp()
            start_ts.FromDatetime(start_time)
            end_ts = Timestamp()
            end_ts.FromDatetime(end_time)
            
            # Map format string to enum
            format_map = {
                'JSON': 0,
                'JSONL': 1,
                'CSV': 2,
                'TEXT': 3
            }
            export_format = format_map.get(format.upper(), 1)
            
            request = loki_service_pb2.ExportLogsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                query=query,
                start=start_ts,
                end=end_ts,
                format=export_format,
                chunk_size=chunk_size
            )

            print(f"✅ [Loki] Exporting logs...")
            
            for response in self.stub.ExportLogs(request):
                if callback:
                    callback(response.export_id, response.data, response.processed_entries,
                            response.total_entries, response.complete)
                else:
                    print(f"   📦 Chunk: {response.processed_entries}/{response.total_entries} entries")
                
                if response.complete:
                    print(f"   ✅ Export complete: {response.export_id}")
                    break

        except Exception as e:
            self.handle_error(e, "Export logs")

    def get_export_status(self, export_id: str) -> Optional[Dict]:
        """Get export task status"""
        try:
            self._ensure_connected()
            request = loki_service_pb2.GetExportStatusRequest(
                user_id=self.user_id,
                export_id=export_id
            )

            response = self.stub.GetExportStatus(request)

            status = {
                'export_id': response.export_id,
                'status': response.status,
                'total_entries': response.total_entries,
                'exported_entries': response.exported_entries,
                'progress_percentage': response.progress_percentage,
                'created_at': response.created_at,
                'completed_at': response.completed_at,
                'error_message': response.error_message
            }
            
            print(f"✅ [Loki] Export status: {response.status} ({response.progress_percentage}%)")
            return status

        except Exception as e:
            return self.handle_error(e, "Get export status")

    def get_statistics(self, start_time: datetime = None, end_time: datetime = None) -> Optional[Dict]:
        """Get Loki statistics"""
        try:
            self._ensure_connected()
            
            start_ts = None
            end_ts = None
            if start_time:
                start_ts = Timestamp()
                start_ts.FromDatetime(start_time)
            if end_time:
                end_ts = Timestamp()
                end_ts.FromDatetime(end_time)
            
            request = loki_service_pb2.GetStatisticsRequest(
                user_id=self.user_id,
                organization_id=self.organization_id,
                start=start_ts,
                end=end_ts
            )

            response = self.stub.GetStatistics(request)

            stats = {
                'total_entries': response.total_entries,
                'total_bytes': response.total_bytes,
                'streams_count': response.streams_count,
                'entries_by_service': dict(response.entries_by_service),
                'entries_by_level': dict(response.entries_by_level),
                'ingestion_rate': [(p.timestamp, p.value) for p in response.ingestion_rate],
                'top_services': list(response.top_services),
                'metadata': dict(response.metadata)
            }
            
            print(f"✅ [Loki] Statistics retrieved:")
            print(f"   Total entries: {response.total_entries}")
            print(f"   Total bytes: {response.total_bytes/1024/1024:.2f}MB")
            print(f"   Streams: {response.streams_count}")
            return stats

        except Exception as e:
            return self.handle_error(e, "Get statistics")

    def get_user_quota(self) -> Optional[Dict]:
        """Get user quota information"""
        try:
            self._ensure_connected()
            request = loki_service_pb2.GetUserQuotaRequest(
                user_id=self.user_id,
                organization_id=self.organization_id
            )

            response = self.stub.GetUserQuota(request)

            print(f"✅ [Loki] Quota: today used {response.today_used}/{response.daily_limit}")
            print(f"   Storage: {response.storage_used_bytes / 1024 / 1024:.2f}MB / {response.storage_limit_bytes / 1024 / 1024:.2f}MB")
            print(f"   Retention: {response.retention_days} days")

            return {
                'daily_limit': response.daily_limit,
                'today_used': response.today_used,
                'storage_limit_bytes': response.storage_limit_bytes,
                'storage_used_bytes': response.storage_used_bytes,
                'retention_days': response.retention_days,
                'quota_exceeded': response.quota_exceeded
            }

        except Exception as e:
            return self.handle_error(e, "Get user quota")


# Convenience usage example
if __name__ == '__main__':
    with LokiClient(host='localhost', port=50054, user_id='test_user') as client:
        # Health check
        client.health_check()

        # Push simple log
        client.push_simple_log(
            message="Application started successfully",
            service="my-service",
            level="INFO"
        )

        # Push log with labels
        client.push_log(
            message="User login successful",
            labels={
                "action": "login",
                "user": "john_doe",
                "ip": "192.168.1.1"
            }
        )

        # Batch push
        logs = [
            {"message": "Processing request 1", "labels": {"request_id": "req-1"}},
            {"message": "Processing request 2", "labels": {"request_id": "req-2"}},
            {"message": "Processing request 3", "labels": {"request_id": "req-3"}}
        ]
        client.push_log_batch(logs)

        # Query logs
        results = client.query_logs(query='{service="my-service"}', limit=10)
        print(f"Query results: {results}")

        # Get quota info
        quota = client.get_user_quota()
        print(f"Quota info: {quota}")
